@@ -22,6 +22,14 @@
 
 #include "config.h"
 
+#if __APPLE__
+// In Mac OS X 10.5 and later trying to use the daemon function gives a “‘daemon’ is deprecated”
+// error, which prevents compilation because we build with "-Werror".
+// Since this is supposed to be portable cross-platform code, we don't care that daemon is
+// deprecated on Mac OS X 10.5, so we use this preprocessor trick to eliminate the error message.
+#define daemon yes_we_know_that_daemon_is_deprecated_in_os_x_10_5_thankyou
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdbool.h>
@@ -41,7 +49,11 @@
 #include <unistd.h>
 #include <fcntl.h>
 
-#include <syslog.h>
+#ifndef WIN32
+#   include <syslog.h>
+#else
+#   define O_NONBLOCK (0) /* O_NONBLOCK does not exist for Windows */
+#endif
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -64,11 +76,18 @@
 #   include "tcp.h"
 #endif
 
+#if __APPLE__
+#undef daemon
+extern int daemon(int, int);
+#endif
+
 #define FIFO_THRESHOLD_SIZE (400 * 1024 * 1024) /* threshold in bytes */
 #define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
 
+#ifdef HAVE_SYS_SOCKET_H
 static const int   i_summary_mode[] = { SUM_BANDWIDTH, SUM_TABLE, SUM_PACKET, SUM_WIRE };
 static const char *psz_summary_mode[] = { "bandwidth", "table", "packet", "wire" };
+#endif
 
 /*****************************************************************************
  *
@@ -118,14 +137,16 @@ static void usage(void)
     printf("                         table  = tables and descriptors\n");
     printf("                         packet = decode packets and print structs\n");
 //    printf("                         wire = print arrival time per packet (wireshark like)\n");
-    printf("      --summary-file   : file to write summary information to (default: stdout)\n");
-    printf("      --summary-period : refresh summary file every n milliseconds (default: 1000ms)\n");
+    printf(" -j | --summary-file   : file to write summary information to (default: stdout)\n");
+    printf(" -p | --summary-period : refresh summary file every n milliseconds (default: 1000ms)\n");
 #endif
     exit(EXIT_FAILURE);
 }
 
 /* Logging */
+#ifndef WIN32
 static int log_level[] = { LOG_ERR, LOG_WARNING, LOG_INFO, LOG_DEBUG };
+#endif
 static const char *psz_level[] = { "ERROR", "WARNING", "INFO", "DEBUG" };
 
 static void libdvbpsi_log(void *data, const int level, const char *format, ...)
@@ -154,9 +175,11 @@ static void libdvbpsi_log(void *data, const int level, const char *format, ...)
         free(msg);
         return;
     }
+#ifndef WIN32
     if (param->b_monitor)
         syslog(log_level[level], "%s", msg);
     else
+#endif
         fprintf(stderr, "%s: %s", psz_level[level], msg);
     free(msg);
 }
@@ -196,6 +219,7 @@ static void params_free(params_t *param)
     free(param->input);
     free(param->output);
     free(param->summary.file);
+    free(param);
     param = NULL;
 }
 
@@ -203,15 +227,15 @@ static void params_free(params_t *param)
 static void dvbinfo_close(params_t *param)
 {
 #ifdef HAVE_SYS_SOCKET_H
-    if (param->input && param->b_udp)
+    if (param->input && param->b_udp && (param->fd_in >= 0))
         udp_close(param->fd_in);
-    else if (param->input && param->b_tcp)
+    else if (param->input && param->b_tcp && (param->fd_in >= 0))
         tcp_close(param->fd_in);
     else
 #endif
-    if (param->input)
+    if (param->input && (param->fd_in >= 0))
         close(param->fd_in);
-    if (param->output)
+    if (param->output && (param->fd_out >= 0))
         close(param->fd_out);
 }
 
@@ -357,14 +381,14 @@ static int dvbinfo_process(dvbinfo_capture_t *capture)
 
         if (param->output)
         {
-            size_t size = param->pf_write(param->fd_out, buffer->p_data, buffer->i_size);
+            ssize_t size = param->pf_write(param->fd_out, buffer->p_data, buffer->i_size);
             if (size < 0) /* error writing */
             {
                 libdvbpsi_log(param, DVBINFO_LOG_ERROR,
                               "error (%d) writting to %s", errno, param->output);
                 break;
             }
-            else if (size < buffer->i_size) /* short writting disk full? */
+            else if ((size_t)size < buffer->i_size) /* short writting disk full? */
             {
                 libdvbpsi_log(param, DVBINFO_LOG_ERROR,
                               "error writting to %s (disk full?)", param->output);
@@ -387,7 +411,13 @@ static int dvbinfo_process(dvbinfo_capture_t *capture)
                     fflush(fd);
                     fclose(fd);
                     unlink(param->summary.file);
-                    rename(psz_temp, param->summary.file);
+                    int ret = rename(psz_temp, param->summary.file);
+                    if (ret < 0)
+                    {
+                        libdvbpsi_log(param, DVBINFO_LOG_ERROR,
+                                      "failed renming summary file (disabling summary logging)\n");
+                        param->b_summary = false;
+                    }
                 }
                 else
                 {
@@ -477,7 +507,7 @@ int main(int argc, char **pp_argv)
         { NULL, 0, NULL, 0 }
     };
 #ifdef HAVE_SYS_SOCKET_H
-    while ((c = getopt_long(argc, pp_argv, "a:d:f:i:ho:ms:tu", long_options, NULL)) != -1)
+    while ((c = getopt_long(argc, pp_argv, "a:d:f:i:j:ho:p:ms:tu", long_options, NULL)) != -1)
 #else
     while ((c = getopt_long(argc, pp_argv, "d:f:h", long_options, NULL)) != -1)
 #endif
@@ -514,7 +544,6 @@ int main(int argc, char **pp_argv)
 
 #ifdef HAVE_SYS_SOCKET_H
             case 'a':
-            {
                 if (optarg)
                 {
                     if (asprintf(&param->mcast_interface, "%s", optarg) < 0)
@@ -523,7 +552,8 @@ int main(int argc, char **pp_argv)
                         usage();
                     }
                 }
-            }
+                break;
+
             case 'i':
                 if (optarg)
                 {
@@ -573,17 +603,26 @@ int main(int argc, char **pp_argv)
             /* - Statistics */
             case 's':
             {
-                param->b_summary = true;
-                ssize_t size = ARRAY_SIZE(psz_summary_mode);
-                for (ssize_t i = 0; i < size; i++)
+                if (optarg)
                 {
-                    printf("summary mode %s\n", psz_summary_mode[i]);
-                    if (strncmp(optarg, psz_summary_mode[i], strlen(psz_summary_mode[i])) == 0)
+                    ssize_t size = ARRAY_SIZE(psz_summary_mode);
+                    for (ssize_t i = 0; i < size; i++)
                     {
-                        param->summary.mode = i_summary_mode[i];
-                        break;
+                        printf("summary mode %s\n", psz_summary_mode[i]);
+                        if (strncmp(optarg, psz_summary_mode[i], strlen(psz_summary_mode[i])) == 0)
+                        {
+                            param->summary.mode = i_summary_mode[i];
+                            param->b_summary = true;
+                            break;
+                        }
                     }
                 }
+                if (!param->b_summary)
+                {
+                    fprintf(stderr, "Option --summary has invalid content %s\n", optarg);
+                    params_free(param);
+                    usage();
+                }
                 break;
             }
             case 'j':
@@ -598,9 +637,9 @@ int main(int argc, char **pp_argv)
                 break;
 
             case 'p':
+                if (optarg)
                 {
-                    char *end = NULL;
-                    param->summary.period = strtoll(optarg, &end, 10);
+                    param->summary.period = strtoll(optarg, NULL, 10);
                     if (((errno == ERANGE) &&
                             ((param->summary.period == LLONG_MIN) ||
                              (param->summary.period == LLONG_MAX))) ||
